/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  latexWriter.hpp
///  \brief Useful tool for debugging the algorithm. We write a latex file showing the backtrack tree.
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   2010-09-29
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#ifndef LATEXWRITER_HPP_
#define LATEXWRITER_HPP_

#include "node.hpp"

class LatexWriterOff;
class LatexWriter;

typedef LatexWriterOff LATEXCLASS;  ///< no latex file will be written
//typedef LatexWriter LATEXCLASS;   ///< a latex file will be written to the  file defined by LATEXOUT
#define LATEXOUT "debug_file.tex" ///< change this if necessary

class LatexWriterOff 
{
public:
	LatexWriterOff(void) {};
	~LatexWriterOff() {};
	void finish_level(bool b) {};
	template<typename RING, AlgTypeE ALGTYPE>
	void newNode(const Node<RING, ALGTYPE>& node, std::string text) {};
	void prunedNode(const std::string& text) {}
};

class LatexWriter : public std::fstream 
{
private:
	int inf_level;
	
public:
	LatexWriter(void) : inf_level(2)
	{
		this->open(LATEXOUT, std::fstream::out);
		(*this)<<"\\documentclass[a4paper, landscape, german]{article}"<<std::endl;
		(*this)<<"\\usepackage[german]{babel}"<<std::endl;
		(*this)<<"\\usepackage{array}"<<std::endl;
		(*this)<<"\\usepackage{pst-all}"<<std::endl;
		(*this)<<"\\usepackage{pstricks-add}"<<std::endl;
		(*this)<<"\\pagestyle{empty}"<<std::endl;
		(*this)<<"\\begin{document}"<<std::endl;
		(*this)<<"\\psset{xunit=0.5cm, yunit=0.5cm, runit=0.5cm, levelsep = 5cm, treefit=loose, arrows = ->}"<<std::endl;
		(*this)<<"\\begin{pspicture}(23,24)"<<std::endl<<"\\rput(0,0){"<<std::endl;
		(*this)<<"\\rotateright{"<<std::endl<<"\\psscalebox{0.05}{"<<std::endl;
	}
	
	~LatexWriter()
	{
		(*this)<<"}}}"<<std::endl<<"\\end{pspicture}"<<std::endl<<"\\end{document}"<<std::endl;
	}
	
	
	void finish_level(bool b)
	{
		if(!b)
		{
			(*this)<<"}";
		}
	}
	
	void prunedNode(const std::string& text)
	{
	    (*this)<<"\\Toval{"<<text<<"}"<<std::endl;
	}

	template<typename RING, AlgTypeE ALGTYPE>
	void newNode(const Node<RING, ALGTYPE>& node,  std::string text);
};

template<typename RING, AlgTypeE ALGTYPE>
void LatexWriter::newNode(const Node<RING, ALGTYPE>& node, std::string text)
{
	(*this)<<"\\pstree{"<<std::endl;
	if(inf_level==1)
	{
		if(text.empty())
		{
			(*this)<<"\\Tdot"<<std::endl;
		}
		else
		{
			(*this)<<"\\Toval{"<<text<<"}"<<std::endl;
		}
	}

	if(inf_level==2)
	{
		(*this)<<"\\Tr{"<<std::endl;
		(*this)<<"\\begin{tabular}{c";
		
		OrdPartition blocks(node.getBlockSignature());
		
		for(unsigned int i=1; i<node.getCols(); ++i)
		{
			if(blocks[i])
			{
				(*this)<<"|";
			}
			(*this)<<"c";
		}
		(*this)<<"}"<<std::endl;


		unsigned int lastColumn = node.getCols()-1;
		for(unsigned int i=0; i<node.getRows(); ++i)
		{
			for(unsigned int j=0; j<node.getCols(); ++j)
			{
				if(node.getInnerStab().m_isFixedCol[j])
				{
					(*this)<<"\\color{red} ";
				}
				(*this)<<node.getEntry(i,j);

				if(j==lastColumn)
				{
				    (*this)<<"\\\\"<<std::endl;
				}
				else
				{
				    (*this)<<"&";
				}
			}

		}
		if(!text.empty() && text.compare("Root")!=0)
		{
			(*this)<<"\\multicolumn{"<<node.getCols()<<"}{c}{"<<text<<"}\\\\"<<std::endl;
		}
		(*this)<<"\\end{tabular}"<<std::endl;
		(*this)<<"}"<<std::endl;
	}

	if(inf_level>0 && text.compare("Root")!=0)
	{
		//    Permutation::FROM_ONE = 1;
		(*this)<<"\\tlput{";
		node.getPermutation().print(*this,0);
		(*this)<<"}"<<std::endl;
//		Permutation::FROM_ONE = 0;
	}
	(*this)<<"}{"<<std::endl;
}

#endif /*LATEXWRITER_HPP_*/
