/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  matrix.cpp
///  \brief 
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   05.10.2010
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matrix.hpp"
#include "helpfunction.hpp"
#include <set>

//template<>
//std::vector<Codeword<Field> >& Matrix<Field>::getCodewords(std::vector<Codeword<Field> >& res, bool remove) const
//{
//    res.clear();
//    CodewordEnumerator<Field> enumerator(std::vector<unsigned int>(1, getRows()), *this);
//    Field::MyWeight maxWeight(getRows() + 1); //  todo: the codewords which have no zero entries are not interesting, change this. See also: Node::minWordRefine
//    std::map<Field::MyWeight, std::vector<Codeword<Field> > > weight2Words;
//    unsigned int storedWords = 0;
//
//    while (++enumerator)
//    {
//        if (!remove)
//        {
//            res.push_back(*enumerator);
//        }
//        else
//        {
//            Field::MyWeight w(enumerator->getWeight());
//            if (w < maxWeight)
//            {
//                if (enumerator.isProjPointRepresentative())// only one representative for each 1-dim subspace
//                {
//                    weight2Words[w].push_back(*enumerator);
//                    ++storedWords;
//                    if (weight2Words.size() > 1 && (storedWords >= MAXIMAL_INIT_WORKSIZE || weight2Words.size()
//                            > MAXIMAL_NR_OF_WEIGHTS))
//                    {
//                        std::map<Field::MyWeight, std::vector<Codeword<Field> > >::reverse_iterator rit(
//                                weight2Words.rbegin());
//                        storedWords -= rit->second.size();
//                        maxWeight = rit->first;
//                        weight2Words.erase(rit->first);
//                    }
//                }
//            }
//        }
//    }
//
//    if (remove)
//    {
//        std::map<Field::MyWeight, std::vector<Codeword<Field> > >::iterator mapIter(weight2Words.begin());
//        std::map<Field::MyWeight, std::vector<Codeword<Field> > >::iterator mapIterEnd(weight2Words.end());
//        for (; mapIter != mapIterEnd; ++mapIter)
//        {
////            std::cout<<mapIter->first<<"*"<<mapIter->second.size()<<"  ";
//            unsigned int oldsize = res.size();
//            res.resize(oldsize + mapIter->second.size());
//            std::copy(mapIter->second.begin(), mapIter->second.end(), res.begin() + oldsize);
//        }
////        std::cout<<std::endl;
//    }
//    return res;
//}

//template<>
//std::vector<Codeword<Z4> >& Matrix<Z4>::getCodewords(std::vector<Codeword<Z4> >& res, bool remove) const
//{
//    Matrix<Z4> copy(*this);
//    unsigned int k0 = copy.toOrderedBasis(false, NULL, NULL);
//    Matrix<Z4> t = copy.transpose();
//
//    return t.getColumnSpan(k0, res, remove);
//}

template<>
void Matrix<Field>::applyFrobenius(unsigned int frob_pow, const std::vector<bool>* const isfixedCol)
{
    if (frob_pow == 0)
    {
        return;
    }

    const std::vector<unsigned int>& img = Field::getFrobenius(frob_pow);
    std::vector<Vec<Field> >::iterator colIter(m_entries.begin());
    std::vector<Vec<Field> >::iterator colIterEnd(m_entries.end());

    std::vector<bool>::const_iterator fixIter;
    std::vector<bool> dummy_vec;

    if (isfixedCol == NULL)
    {
        dummy_vec.resize(m_n, false);
        fixIter = dummy_vec.begin();
    }
    else
    {
        fixIter = isfixedCol->begin();
    }

    for (; colIter < colIterEnd; ++colIter, ++fixIter)
    {
        if (!(*fixIter))
        {
            Vec<Field>::iterator iter(colIter->begin());
            Vec<Field>::const_iterator iterEnd(colIter->end());

            for (; iter < iterEnd; ++iter)
            {
                *iter = Element<Field> (img[iter->getIndex()]);
            }
        }
    }
}

// Span(G1) contains Span(G2), we want to have a basis of W (as matrix again) such that  Span(G1) =  Span(G2) \oplus W
template<>
unsigned int Matrix<Field>::complement(const Matrix<Field>& G1, const Matrix<Field>& G2, Matrix<Field>& res)
{
    Matrix<Field> all;
    VerticalJoin(G2, G1, all);
    unsigned int pivot_col = 0, pivot_row_upper_part = 0, pivot_row_lower_part = G2.getRows();

    while (true)
    {
        std::pair<unsigned int, unsigned int> res(all.m_entries[pivot_col].height(pivot_row_upper_part));
        if (res.first == 0)
        {
            if( res.second < G2.getRows() )
            {
                all.gaussElStep(pivot_row_upper_part, pivot_col, false, NULL);
                ++pivot_row_upper_part;
            }
            else
            {
                res = all.m_entries[pivot_col].height(pivot_row_lower_part);
                if(res.first ==0)
                {
                    all.gaussElStep(pivot_row_lower_part, pivot_col, false, NULL);
                    ++pivot_row_lower_part;
                }
            }
        }
    }

    res.rowSubmatrix( all, G2.getRows(), pivot_row_lower_part- G2.getRows());
    return pivot_row_lower_part- G2.getRows();
}

